#!/usr/bin/env python

from grammarLexer import GrammarLexer
from grammarLexer import T_TOKEN
from grammar import Grammar
from terminal import Terminal
from nonTerminal import NonTerminal

class GrammarParser:
    __lxr = GrammarLexer()

    __grammar = {}

    def parse(self, grammarFile):
        lines = grammarFile.readlines()

        for line in lines:

            self.__lxr.processLine(line)

            tokenLine = self.__lxr.getParsedLine()


            if self.__lineIsValid(tokenLine):
                leftSide = tokenLine.popleft().getLexeme()
                tokenLine.popleft()
                rightSide = []

                while len(tokenLine) > 0:
                    rightSide.append(tokenLine.popleft())

                if self.__isNewNonTerm(leftSide):
                    self.__grammar[leftSide] = []

                self.__grammar[leftSide].append(rightSide)

    def parseGrm(self, grammarFile):
        grm = Grammar()
        lines = grammarFile.readlines()

        for line in lines:
            self.__lxr.processLine(line)
            tokenLine = self.__lxr.getParsedLine()

            if self.__lineIsValid(tokenLine):
                leftSide = NonTerminal(tokenLine.popleft().getLexeme())
                tokenLine.popleft()

                rightSide = []
                tmpToken = None

                while len(tokenLine) > 0:
                    tmpToken = tokenLine.popleft()

                    if tmpToken.getClass() == T_TOKEN.NON_TERMINAL:
                        rightSide.append(NonTerminal(tmpToken.getLexeme()))
                    elif tmpToken.getClass() == T_TOKEN.TERMINAL:
                        lexeme = tmpToken.getLexeme();

                        if lexeme == "~":
                            lexeme = None;

                        rightSide.append(Terminal(lexeme))

                grm.addProduction(leftSide, rightSide)

        return grm


    def __lineIsValid(self, tokenLine):
        if len(tokenLine) < 3:
            return False

        if tokenLine[0].getClass() != T_TOKEN.NON_TERMINAL:
            return False

        if tokenLine[1].getClass() != T_TOKEN.ARROW:
            return False;

        return True;

    def __isNewNonTerm(self, token):
        if self.__grammar.get(token) == None:
            return True
        else:
            return False

    def printGrammar(self):
        for left in self.__grammar:
            self.__printRow(left, self.__grammar[left]);


    def __printRow(self, left, right):
        line = left + " -> "

        for i in range(len(right) - 1):
            for token in right[i]:
                line += str(token.getClass()) + "(" + token.getLexeme() + ")"
            line += " | "

        for token in right[len(right) -1]:
            line += str(token.getClass()) + "(" + token.getLexeme() + ")"

        print(line)
